from .bed_rush import *
from .bedwalls import *
from .bedwars_late import *
from .bedwars_mega import *
from .bedwars_normal import *
from .bedwars_rush import *
from .bow_fight import *
from .bridge_fight import *
from .flat_fight import *
from .four_way_bridges import *
from .ground_fight import *
from .ladder_fight import *
from .mini_wars import *
from .obstacles import *
from .party_games import *
from .pearl_fight import *
from .ranked_fours_practice import *
from .resource_collect import *
from .resource_old_collect import *
from .stick_fight import *
from .sumo_duels import *
from .sumo import *
from .void_fight import *
