"""Voxyl API Wrapper

A  simple python wrapper for the Voxyl API

"""

__title__ = "voxapiwrapper"
__author__ = "_lightninq"
__license__ = "MIT"
__copyright__ = "Copyright 2022 BWP-Stats"
__version__ = "1.0.0"

from .errors import *
from .client import Client
